#ifndef LINEVEC_H
#define LINEVEC_H

#ifndef  _USE_MATH_DEFINES
#define  _USE_MATH_DEFINES

#include <math.h>
#include "KeyValue.h"

class LineVec {
  public:
  Point origin, destn; // The end points of the line
  LineVec() { }
  LineVec(Point _origin, Point _destn){
    if(_origin.equals(_destn)) {
      fprintf(stderr,"origin and destn are same! \n");
    }
    origin.copyFrom(_origin);
    destn.copyFrom(_destn);
  } // End LineVec(Point, Point)
  
  /** Return the length of the vector */
  double length() { return Point::distance(origin, destn); }
  
  /** Copy from the given vector */
  void copyFrom(const LineVec *lv){ origin = lv->origin; destn = lv->destn;}
  
  /** Return the angle with the x-axis -- as measured when the
      positive x-axis vector is rotated clock-wise to align with this
      vector.  Returns the angle in RADIANS.  Range: [0,2*pi) */
  double angleWithXAxis() {
    double diffy = (destn.y - origin.y);
    double dist = length();
    if(dist == 0) {
      fprintf(stderr,"angleWithXAxis: vector length is 0 \n");
    }
    double angle1 = asin(diffy/dist);
    if(angle1 >= 0) {
      if(destn.x >= origin.x) { return angle1; } // I quadrant
      else { return (M_PI - angle1); } // II quadrant
    } else {
      angle1 = -angle1; // to make the computations below easier to understand
      if(destn.x >= origin.x) { return 2*M_PI - angle1; } // IV quadrant
      else { return M_PI + angle1; } // III quadrant
    }
  } // End angleWithXAxis()

  /** Return the angle with the given vector.  Return value is in
      radians, and ranges [0,pi]. */
  double angle(LineVec *lv2){
    return LineVec::angle(this, lv2);
  } // End angle(LineVec lv2)
                                                                                
  /** Return the angle between the given vectors.  Return value is in
      radians, and ranges [0,pi]. */
  static double angle(LineVec *lv1, LineVec *lv2){
    double a1 = lv1->angleWithXAxis();
    double a2 = lv2->angleWithXAxis();
    double adiff = fabs(a1-a2);
    return (adiff > M_PI) ? (2*M_PI - adiff) : adiff;
  } // End angle(LineVec lv1, LineVec lv2)

}; // End class LineVec
typedef class LineVec LineVec_t;

#endif // _USE_MATH_DEFINES
#endif // LINEVEC_H
