/** Program to calculate bandwidth of TCP connection. */

#include <stdio.h>
#include <unistd.h>

#include "Util.h"

#define TCP_PORT 1200
#define SERVER_MODE 0
#define CLIENT_MODE 1

void usage(const char *progname) {
  fprintf(stderr, "Usage: %s <-s/-c> <remote-port> <remote-host> <numbytes> <bidir=0/1>\n", progname);
} // End usage()

void server(int port, int bidir, int numbytes) {

  int s = sock_and_listen(port);
  if(s == -1) { perror("sock_and_listen"); exit(100); }

  fprintf(stderr, "Listening on port %d\n\a", port);

  struct sockaddr_in cl_addr;
  int cl_addr_size = sizeof(cl_addr);
  int cs = accept(s, (struct sockaddr*)&cl_addr, (socklen_t*)&cl_addr_size);
  char cl_str[1024];
  ipPrint(cl_str, cl_addr.sin_addr);
  fprintf(stderr, "Connection from %s:%d\n\a", cl_str, ntohs(cl_addr.sin_port));

  struct timeval start;
  gettimeofday(&start, NULL);
  int totRecd = 0;
  int totSent = 0;

  // numbytes not used!

  for(;;) {

    char buf[4096];

    if(bidir) {
      int num = recv(cs, buf, sizeof(buf), 0);
      if(num <= 0) { break; }
      totRecd += num;
    }

    int num_sent = send(cs, buf, sizeof(buf), 0);
    if(num_sent < 0) {
      perror("send");
      break;
    }
    totSent += num_sent;

  } // End for(;;)
  double time = now(start)/1000.0; // in seconds
  fprintf(stderr, "Num bytes recd = %d\n", totRecd);
  fprintf(stderr, "Time taken = %0.3f\n", time);
  fprintf(stderr, "Throughput = %0.3f M-Bytes-per-sec\n", 1.0*totRecd*1e-6/time);
  fprintf(stderr, "Throughput = %0.3f M-bits-per-sec\n", 8.0*totRecd*1e-6/time);
  fprintf(stderr, "Num bytes sent = %d\n", totSent);

  fprintf(stderr, "\n\a");

  close(cs);
  close(s);

} // End server()

void client(const char *host, int port, int bidir, int numbytes) {

  struct in_addr server_addr = nslookup(host);
  fprintf(stderr, "Trying to connect to %s:%d\n",
	  ip2str(server_addr).cstr(), port);
  int s = sock_and_connect(host, port);
  if(s == -1) { perror("client"); exit(200); }

  struct timeval start;
  gettimeofday(&start, NULL);
  int totRecd = 0;
  int totSent = 0;

  while(totRecd < numbytes) {
    char buf[4096];

    if(bidir) {
      int num_sent = send(s, buf, sizeof(buf), 0);
      if(num_sent < 0) {
	perror("send");
	break;
      }
      totSent += num_sent;
    }

    int num = recv(s, buf, sizeof(buf), 0);
    if(num <= 0) { break; }
    totRecd += num;

  } // End for(;;)

  double time = now(start)/1000.0; // in seconds
  fprintf(stderr, "Num bytes recd = %d\n", totRecd);
  fprintf(stderr, "Time taken = %0.3f\n", time);
  fprintf(stderr, "Throughput = %0.3f M-Bytes-per-sec\n", 1.0*totRecd*1e-6/time);
  fprintf(stderr, "Throughput = %0.3f M-bits-per-sec\n", 8.0*totRecd*1e-6/time);
  fprintf(stderr, "Num bytes sent = %d\n", totSent);

  fprintf(stderr, "\n\a");
  close(s);

} // End client()

int main(int argc, char *argv[]) {

  if(argc != 6) { usage(argv[0]); exit(1); }

  int mode = -1;
  if(strcmp(argv[1], "-s") == 0) { mode = SERVER_MODE; }
  else if(strcmp(argv[1], "-c") == 0) { mode = CLIENT_MODE; }
  else { usage(argv[0]); exit(2); }

  char host[1024];
  int port = -1;
  int bidir = -1;

  strcpy(host, "localhost");
  port = atoi(argv[2]);
  strcpy(host, argv[3]);
  int numbytes = atoi(argv[4]);
  if(strcmp(argv[5], "bidir=0") == 0) { bidir = 0; }
  else if (strcmp(argv[5], "bidir=1") == 0) { bidir = 1; }
  else { usage(argv[0]); exit(3); }

  if(mode == SERVER_MODE) { server(port, bidir, numbytes); }
  else { client(host, port, bidir, numbytes); }

  return 0;
} // End main()
