#The script for setup of scenario of cs497 paper

#
#This script will be the base for simulations
#Setting up wireless scenario ,
#
# ======================================================================
# Define options
# ======================================================================



set opt(chan)           Channel/Channel_802_11    		;#Channel Type
set opt(prop)           Propagation/Shadowing  			;# radio-propagation model
set opt(netif)          Phy/WirelessPhy/Wireless_802_11_Phy     ;# network interface type
set opt(mac)            Mac/802_11                		;# MAC type
set opt(ifq)            Queue/DropTail/PriQueue   		;# interface queue type
set opt(ll)             LL                        		;# link layer type
set opt(ant)            Antenna/DirAntenna       		;# antenna model
set opt(mod)		Modulation/BPSK				;	
set opt(ifqlen)         50                        		;# max packet in ifq
set val(nn)             10                         		;# number of mobilenodes
set val(rp)             WLSTATIC                      		;# routing protocol
set val(x)		5000
set val(y)		5000
set val(ni)		1					;# no of interfaces


# Initialize Global Variables
set ns_		[new Simulator]
set tracefd     [open output.tr w]
#set tracefd     [open ~nsuser/results/traces/cs497.tr w]
$ns_ use-newtrace
$ns_ trace-all $tracefd

# set up topography object
set topo       [new Topography]
$topo load_flatgrid $val(x) $val(y)
# Create God
create-god $val(nn)

#configure node defaults
$ns_ node-config -llType $opt(ll) \
		-macType $opt(mac) \
		-ifqType $opt(ifq) \
		-ifqLen $opt(ifqlen) \
		-antType $opt(ant) \
		-propType $opt(prop) \
		-phyType $opt(netif) \
		-topoInstance $topo \
		-agentTrace ON \
		-routerTrace ON \
		-macTrace ON \
		-movementTrace OFF \
		-numif $val(ni)	\
		-channel [new $opt(chan)]


#Basically setting up a MANET rather than a WLAN

$ns_ node-config -adhocRouting $val(rp)
###############################sroy+ash

###################################################
#                 set other parameters
###################################################


###################################################
#                 Procedures
###################################################

proc create_cbr_connection { from to startTime interval packetSize } {
	global ns_
	set udp0 [new Agent/UDP]
	set src [new Application/Traffic/CBR]
	$udp0 set packetSize_ $packetSize
	$src set packetSize_ $packetSize
	$src set interval_ $interval

	set sink [new Agent/Null]

	$ns_ attach-agent $from $udp0
	$src attach-agent $udp0
	$ns_ attach-agent $to $sink

	$ns_ connect $udp0 $sink
	$ns_ at $startTime "$src start"
	return $udp0
}

proc create_tcp_connection { from to startTime } {
	global ns_
	set tcp0 [new Agent/TCP]
	$tcp0 set class_ 2
	set sink0 [new Agent/TCPSink]
	$ns_ attach-agent $from $tcp0
	$ns_ attach-agent $to $sink0

	set ftp0 [new Application/FTP]
	$ftp0 attach-agent $tcp0

	$ns_ connect $tcp0 $sink0
	$ns_ at $startTime "$ftp0 start"
	return tcp0

}

proc create_node { x y z } {
	global ns_
	Mac/802_11 MAC_RTSThreshold 2239
	Mac/802_11 set dataRate_	11mb
	Mac/802_11 set basicRate_	1mb
	set newnode [$ns_ node]
	$newnode random-motion 0
	$newnode set X_ $x
	$newnode set Y_ $y
	$newnode set Z_ $z

	return $newnode
}

##########################################################
#                 create nodes
##########################################################


#For node no. 1
$ns_ node-config  -numif 1
set node_(0) [create_node 73	838	0]
[$node_(0) set netif_(0)] set channel_number_ 1
[$node_(0) set netif_(0)] set Pt_ 0.01
set a0 [new Antenna/DirAntenna]
$a0 setAngle	280
$a0 setWidth	60
$a0 setType	0
[$node_(0) set netif_(0)] dir-antenna $a0

#For node no. 2
$ns_ node-config  -numif 3
set node_(1) [create_node 585	435	0]
[$node_(1) set netif_(0)] set channel_number_ 1
[$node_(1) set netif_(0)] set Pt_ 0.01
[$node_(1) set netif_(1)] set channel_number_ 1
[$node_(1) set netif_(1)] set Pt_ 0.01
[$node_(1) set netif_(2)] set channel_number_ 1
[$node_(1) set netif_(2)] set Pt_ 0.01
set a10 [new Antenna/DirAntenna]
$a10 setAngle	5
$a10 setWidth	60
$a10 setType	0
[$node_(1) set netif_(0)] dir-antenna $a10
set a11 [new Antenna/DirAntenna]
$a11 setAngle	70
$a11 setWidth	60
$a11 setType	0
[$node_(1) set netif_(1)] dir-antenna $a11
set a12 [new Antenna/DirAntenna]
$a12 setAngle	200
$a12 setWidth	60
$a12 setType	0
[$node_(1) set netif_(2)] dir-antenna $a12


#For node no. 3
$ns_ node-config  -numif 2
set node_(2) [create_node 373	472	0]
[$node_(2) set netif_(0)] set channel_number_ 1
[$node_(2) set netif_(0)] set Pt_ 0.01
[$node_(2) set netif_(1)] set channel_number_ 1
[$node_(2) set netif_(1)] set Pt_ 0.01
set a20 [new Antenna/DirAntenna]
$a20 setAngle	110
$a20 setWidth	60
$a20 setType	0
[$node_(2) set netif_(0)] dir-antenna $a20
set a21 [new Antenna/DirAntenna]
$a21 setAngle	290
$a21 setWidth	60
$a21 setType	0
[$node_(2) set netif_(1)] dir-antenna $a21


#For node no. 4
$ns_ node-config  -numif 3
set node_(3) [create_node 523	367	0]
[$node_(3) set netif_(0)] set channel_number_ 1
[$node_(3) set netif_(0)] set Pt_ 0.01
[$node_(3) set netif_(1)] set channel_number_ 1
[$node_(3) set netif_(1)] set Pt_ 0.01
[$node_(3) set netif_(2)] set channel_number_ 1
[$node_(3) set netif_(2)] set Pt_ 0.01
set a30 [new Antenna/DirAntenna]
$a30 setAngle	10
$a30 setWidth	60
$a30 setType	0
[$node_(3) set netif_(0)] dir-antenna $a30
set a31 [new Antenna/DirAntenna]
$a31 setAngle	110
$a31 setWidth	60
$a31 setType	0
[$node_(3) set netif_(1)] dir-antenna $a31
set a32 [new Antenna/DirAntenna]
$a32 setAngle	280
$a32 setWidth	60
$a32 setType	0
[$node_(3) set netif_(2)] dir-antenna $a32


#For node no. 5
$ns_ node-config  -numif 1
set node_(4) [create_node 910	575	0]
[$node_(4) set netif_(0)] set channel_number_ 1
[$node_(4) set netif_(0)] set Pt_ 0.01
set a40 [new Antenna/DirAntenna]
$a40 setAngle	190
$a40 setWidth	60
$a40 setType	0
[$node_(4) set netif_(0)] dir-antenna $a40

#For node no. 6
$ns_ node-config  -numif 1
set node_(5) [create_node 910	117	0]
[$node_(5) set netif_(0)] set channel_number_ 1
[$node_(5) set netif_(0)] set Pt_ 0.01
set a50 [new Antenna/DirAntenna]
$a50 setAngle	130
$a50 setWidth	60
$a50 setType	0
[$node_(5) set netif_(0)] dir-antenna $a50

#For node no. 7
$ns_ node-config  -numif 2
set node_(6) [create_node 585	757	0]
[$node_(6) set netif_(0)] set channel_number_ 1
[$node_(6) set netif_(0)] set Pt_ 0.01
[$node_(6) set netif_(1)] set channel_number_ 1
[$node_(6) set netif_(1)] set Pt_ 0.01
set a60 [new Antenna/DirAntenna]
$a60 setAngle	30
$a60 setWidth	60
$a60 setType	0
[$node_(6) set netif_(0)] dir-antenna $a60
set a61 [new Antenna/DirAntenna]
$a61 setAngle	250
$a61 setWidth	60
$a61 setType	0
[$node_(6) set netif_(1)] dir-antenna $a61


#For node no. 8
$ns_ node-config  -numif 2
set node_(7) [create_node 263	611	0]
[$node_(7) set netif_(0)] set channel_number_ 1
[$node_(7) set netif_(0)] set Pt_ 0.01
[$node_(7) set netif_(1)] set channel_number_ 1
[$node_(7) set netif_(1)] set Pt_ 0.01
set a70 [new Antenna/DirAntenna]
$a70 setAngle	110
$a70 setWidth	60
$a70 setType	0
[$node_(7) set netif_(0)] dir-antenna $a70
set a71 [new Antenna/DirAntenna]
$a71 setAngle	290
$a71 setWidth	60
$a71 setType	0
[$node_(7) set netif_(1)] dir-antenna $a71



#For node no. 9
$ns_ node-config  -numif 1
set node_(8) [create_node 674	908	0]
[$node_(8) set netif_(0)] set channel_number_ 1
[$node_(8) set netif_(0)] set Pt_ 0.01
set a80 [new Antenna/DirAntenna]
$a80 setAngle	210
$a80 setWidth	60
$a80 setType	0
[$node_(8) set netif_(0)] dir-antenna $a80

#For node no. 10
$ns_ node-config  -numif 2
set node_(9) [create_node 605	191	0]
[$node_(9) set netif_(0)] set channel_number_ 1
[$node_(9) set netif_(0)] set Pt_ 0.01
[$node_(9) set netif_(1)] set channel_number_ 1
[$node_(9) set netif_(1)] set Pt_ 0.01
set a90 [new Antenna/DirAntenna]
$a90 setAngle	110
$a90 setWidth	60
$a90 setType 	0
[$node_(9) set netif_(0)] dir-antenna $a90
set a91 [new Antenna/DirAntenna]
$a91 setAngle	310
$a91 setWidth	60
$a91 setType	0
[$node_(9) set netif_(1)] dir-antenna $a91

#[$node_(0) set ragent_] addstaticroute <number of hops> <next hop> <destination node> <interface to use>

#Routing strategies
[$node_(0) set ragent_] addstaticroute 3 7 3 0
[$node_(0) set ragent_] addstaticroute 2 7 2 0
[$node_(0) set ragent_] addstaticroute 1 7 7 0

[$node_(1) set ragent_] addstaticroute 1 3 3 2
[$node_(1) set ragent_] addstaticroute 1 4 4 0
[$node_(1) set ragent_] addstaticroute 1 6 6 1
[$node_(1) set ragent_] addstaticroute 2 6 8 1

[$node_(2) set ragent_] addstaticroute 1 3 3 1
[$node_(2) set ragent_] addstaticroute 1 7 7 0
[$node_(2) set ragent_] addstaticroute 2 7 0 0

[$node_(3) set ragent_] addstaticroute 1 1 1 0
[$node_(3) set ragent_] addstaticroute 1 2 2 1
[$node_(3) set ragent_] addstaticroute 1 9 9 3
[$node_(3) set ragent_] addstaticroute 2 1 4 0
[$node_(3) set ragent_] addstaticroute 2 1 6 0
[$node_(3) set ragent_] addstaticroute 2 2 7 1
[$node_(3) set ragent_] addstaticroute 2 9 5 2
[$node_(3) set ragent_] addstaticroute 3 1 8 0
[$node_(3) set ragent_] addstaticroute 3 2 0 1

[$node_(4) set ragent_] addstaticroute 2 1 3 0
[$node_(4) set ragent_] addstaticroute 1 1 1 0

[$node_(5) set ragent_] addstaticroute 2 9 3 0
[$node_(5) set ragent_] addstaticroute 1 9 9 0

[$node_(6) set ragent_] addstaticroute 2 1 3 1
[$node_(6) set ragent_] addstaticroute 1 1 1 1
[$node_(6) set ragent_] addstaticroute 1 8 8 0

[$node_(7) set ragent_] addstaticroute 2 2 3 1
[$node_(7) set ragent_] addstaticroute 1 2 2 1
[$node_(7) set ragent_] addstaticroute 1 0 0 0

[$node_(8) set ragent_] addstaticroute 3 6 3 0
[$node_(8) set ragent_] addstaticroute 2 6 1 0
[$node_(8) set ragent_] addstaticroute 1 6 6 0

[$node_(9) set ragent_] addstaticroute 1 3 3 0
[$node_(9) set ragent_] addstaticroute 1 5 5 1




##########################################################
#                 create connections
##########################################################

set cbr0 [create_cbr_connection $node_(0) $node_(3) 1.0 0.005 1500 ]

set cbr1 [create_cbr_connection $node_(1) $node_(3) 1.1 0.005 1500 ]

set cbr2 [create_cbr_connection $node_(2) $node_(3) 1.2 0.005 1500 ]

set cbr3 [create_cbr_connection $node_(4) $node_(3) 1.4 0.005 1500 ]

set cbr4 [create_cbr_connection $node_(5) $node_(3) 1.5 0.005 1500 ]

set cbr5 [create_cbr_connection $node_(6) $node_(3) 1.6 0.005 1500 ]

set cbr6 [create_cbr_connection $node_(7) $node_(3) 1.7 0.005 1500 ]

set cbr7 [create_cbr_connection $node_(8) $node_(3) 1.8 0.005 1500 ]

set cbr8 [create_cbr_connection $node_(9) $node_(3) 1.9 0.005 1500 ]

###########################################################
# Tell nodes when the simulation ends
###########################################################

for {set i 0} {$i < $val(nn) } {incr i} {
    $ns_ at 10.0 "$node_($i) reset";
}
$ns_ at 10.0 "stop"
$ns_ at 10.01 "puts \"NS EXITING...\" ; $ns_ halt"

proc stop {} {
    global ns_ tracefd
    $ns_ flush-trace
    close $tracefd
}

puts "Starting Simulation..."
$ns_ run


