set qu 2

set opt(nf) 0; #number of flows
set opt(web) 0; #number of web flows
set opt(ftp) 0; #number of ftp flows
set opt(dur) 0.0
set opt(seed) 0.0
set opt(id) -1

set src "bs"
set dest "cr_tr_sink"

set ftp_size 2000; #max num of packets for ftp

proc usage {} {
    global argv0
    puts "Usage: $argv0 \[-seed seed\] \[-web web\] \[-ftp ftp\] \
          \[-dur dur\] \[-id BS_ID\] \n"
}

proc getopt {argc argv} {
	global opt
	lappend optlist web ftp video dur seed

	for {set i 0} {$i < $argc} {incr i} {
		set arg [lindex $argv $i]
		if {[string range $arg 0 0] != "-"} continue

		set name [string range $arg 1 end]
		set opt($name) [lindex $argv [expr $i+1]]
	}
}

getopt $argc $argv
set opt(nf) [expr $opt(web) + $opt(ftp) ]


if {$opt(nf) == 0 || $opt(seed)==0.0 || $opt(id) == -1 } {
    usage
    exit
} 

puts "\n#\#\#\#\#\#\# total conn: $opt(nf), web: $opt(web), \
       ftp:$opt(ftp), dur: $opt(dur), \
       seed: $opt(seed), BS_ID: $opt(id) \#\#\#\#\#\#\#";


#### Random number
set rng [new RNG]
$rng seed $opt(seed)


proc create-web-flows {} {
    global rng opt src dest qu 
    
    for { set i 0} {$i < $opt(web) } {incr i} {
	puts "\n#\#\#\#\#\#\# $i th web flow \#\#\#\#\#\#\#";
	set server_str "wwwServer$opt(id)_($i)"
	set client_str "wwwClient$opt(id)_($i)"
	set fid [expr $qu - 1]
 	puts " set $server_str \[ new WWWServer \$$src\($opt(id)) \
               \$ns $fid \]";
	puts " set $client_str \[ new WWWClient \$$dest \
               \$ns $fid \]";
	puts " \$ns at 0.0 \"\$$client_str start \$$server_str\""
	puts "\$cl\($opt(id)) setqueue $fid $qu"
	puts "\$cl\($opt(id)) setweight $qu \$tcpw"
	set qu [expr $qu + 1]
    }
}



proc create-ftp-flows {} {
    global rng opt src dest qu ftp_size

    for { set i 0} {$i < $opt(ftp) } {incr i} {

	set stime [$rng uniform 0.0 $opt(dur)] 
	set spkt [$rng uniform 100 $ftp_size]
	puts "\n#\#\#\# $i the ftp flow of size $spkt begins at time $stime \
                  \#\#\#\#\#";
	set src_str "tcp$opt(id)_($i)"
	set sink_str "sink$opt(id)_($i)"
	set fid [expr $qu - 1]
	set fid [expr $fid + [expr $opt(id) * 100]]
	set ftp_str "ftp$opt(id)_($i)"
	puts " set $src_str \[new Agent/TCP/Newreno\]"
	puts " set $sink_str \[new Agent/TCPSink/DelAck\]"
	puts " \$ns attach-agent \$$src\($opt(id)) \$$src_str"
	puts " \$ns attach-agent \$$dest \$$sink_str"
	puts " \$$src_str set fid_ $fid "
	puts " \$$sink_str set fid_ $fid "
	puts " \$ns connect \$$src_str \$$sink_str"
	puts " set $ftp_str \[new Application/FTP\]"
	puts " \$$ftp_str attach-agent \$$src_str"
	set spkt [expr round($spkt)]
	puts " \$ns at $stime \"\$$ftp_str produce $spkt\""
	puts "\$cl\($opt(id)) setqueue $fid $qu"
	puts "\$cl\($opt(id)) setweight $qu \$tcpw"
	set qu [expr $qu + 1]
    }
    
}

create-ftp-flows
create-web-flows
